using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using System.Xml;
using MathWorks.MATLAB.NET.Utility;
using MathWorks.MATLAB.NET.Arrays;
using Extrinsic; // Matlab functions

namespace OrthoRef
{
    public partial class ComputeExtrinsicPar : Form
    {

        public double FocalLenght;
        private dsGCP dsGCPCEP = new dsGCP();
        private CExtPar ExtPar = new CExtPar();
 
        public ComputeExtrinsicPar()
        {
            InitializeComponent();
            btnCreate.Enabled = false;
            txbFile.Enabled = true;
            btnOpenFile.Enabled = true;
         }
        
        public CExtPar ExtrinsicParameters
        {
            get
            {
                return ExtPar;
            }
        }

   
        private void btnCreate_Click(object sender, EventArgs e)
        {
            Form1 frmForm1 = new Form1();
            frmForm1.ShowDialog();
            dsGCPCEP = frmForm1.dsGCPData;
        }

        private void btnComputeExtPar_Click(object sender, EventArgs e)
        {
        }

        private void rdbFromImage_CheckedChanged(object sender, EventArgs e)
        {
            if (rdbFromImage.Checked)
            {
                btnCreate.Enabled = true;
                txbFile.Enabled = false;
                btnOpenFile.Enabled = false;
            }
        }
        private void rdbFromFile_CheckedChanged(object sender, EventArgs e)
        {
            if (rdbFromFile.Checked)
            {
                btnCreate.Enabled = false;
                txbFile.Enabled = true;
                btnOpenFile.Enabled = true;
            }

        }

        private void btnOpenFile_Click(object sender, EventArgs e)
        {
            OpenFileDialog dlg = new OpenFileDialog();
            dlg.Filter = "Xml files|*.xml";
            if (dlg.ShowDialog() == DialogResult.OK)
            {
                dsGCPCEP.Clear();
                txbFile.Text = dlg.FileName;
                dsGCPCEP.tlbGCP.ReadXml(dlg.FileName);

             }
        }

        private void btnView_Click(object sender, EventArgs e)
        {
            ViewCPE frmViewCPE = new ViewCPE();
            dsGCP dsGCPEPold = new dsGCP();
            dsGCPEPold = (dsGCP) dsGCPCEP.Copy();
            frmViewCPE.dataGridView1.DataSource = dsGCPCEP;
            frmViewCPE.dataGridView1.DataMember = "tlbGCP";
            frmViewCPE.FileName = txbFile.Text;
            if (frmViewCPE.ShowDialog() == DialogResult.OK)
                txbFile.Text = frmViewCPE.FileName;
            else
            {
                dsGCPCEP.Clear();
                dsGCPCEP = (dsGCP) dsGCPEPold.Copy();
            }

        }



        private void btnDetails_Click(object sender, EventArgs e)
        {
            ViewCPE frmViewCPE = new ViewCPE();
            frmViewCPE.dataGridView1.DataSource = dsGCPCEP;
            frmViewCPE.dataGridView1.DataMember = "tlbGCP";
            frmViewCPE.Show();

        }


        private void ckbOrientation_CheckedChanged(object sender, EventArgs e)
        {
            if (ckbOrientation.Checked)
            {
                Kappa0.Enabled = true;
                Phi0.Enabled = true;
                Omega0.Enabled = true;
            }
            else
            {
                Kappa0.Enabled = false;
                Phi0.Enabled = false;
                Omega0.Enabled = false;
            }
        }

        private void btnOK_Click(object sender, EventArgs e)
        {
            try
            {
                ExtPar.XL = Convert.ToDouble(XL.Text);
                ExtPar.YL = Convert.ToDouble(YL.Text);
                ExtPar.ZL = Convert.ToDouble(ZL.Text);
                ExtPar.Kappa = Convert.ToDouble(Kappa.Text);
                ExtPar.Phi = Convert.ToDouble(Phi.Text);
                ExtPar.Omega = Convert.ToDouble(Omega.Text);
                this.DialogResult = DialogResult.OK;
                this.Close();
 
            }
            catch
            {
                MessageBox.Show("Invalid Camera Orientation Parameters");
            }
 
        }

        private void computeToolStripMenuItem_Click(object sender, EventArgs e)
        {
            MWNumericArray MWNx = null;
            Extrinsicclass MWNExt = new Extrinsicclass();

            ExtPar.dsGCP = dsGCPCEP;

            if(dsGCPCEP.tlbGCP.Count == 0)
            {
                MessageBox.Show("Control points not defined");
                return;
            }
            double[,] x = new double[dsGCPCEP.tlbGCP.Count, 5];
            x = ExtPar.ConvertToMatrix();

            double x0 = 0, y0 = 0, z0 = 0;
            try{
                x0 = Convert.ToDouble(XL0.Text);
                y0 = Convert.ToDouble(YL0.Text);
                z0 = Convert.ToDouble(ZL0.Text);
            }
            catch
            {
                MessageBox.Show("Invalid orientation initial estimates");
                return;
            }

            double kappa;
            double phi;
            double omega;


            if (ckbOrientation.Checked)
            {
                kappa = Convert.ToDouble(Kappa0.Text);
                phi = Convert.ToDouble(Phi0.Text);
                omega = Convert.ToDouble(Kappa0.Text);
            }
            else
            {
                double dx = ExtPar.Mean("GCPX") - x0;
                double dy = ExtPar.Mean("GCPY") - y0;
                double dz = z0 - ExtPar.Mean("GCPZ");

                phi = (Math.Atan2(dy, dx) + Math.PI / 2) * 180 / Math.PI;

 
                omega = (Math.Atan2(Math.Sqrt(Math.Pow(dx, 2) + Math.Pow(dy, 2)), dz)) * 180 / Math.PI;
                kappa = 0;
                Kappa0.Text = kappa.ToString();
                Phi0.Text = phi.ToString();
                Omega0.Text = omega.ToString();
            }


            double[] X0 = { x0, y0, z0, kappa, phi, omega };

            double[] par = { Convert.ToDouble(txbAngularTol.Text), Convert.ToDouble(txbDisplacementTol.Text), Convert.ToDouble(txbIterMax.Text) };

            MWNx = (MWNumericArray)MWNExt.Extrinsic((MWNumericArray)x, (MWNumericArray)FocalLenght, (MWNumericArray)X0, (MWNumericArray)par);
            double[,] dblExtPar = (double[,])MWNx.ToArray(MWArrayComponent.Real);
            ExtPar.XL = dblExtPar[0, 0];
            ExtPar.YL = dblExtPar[1, 0];
            ExtPar.ZL = dblExtPar[2, 0];
            ExtPar.Kappa = dblExtPar[3, 0];
            ExtPar.Phi = dblExtPar[4, 0];
            ExtPar.Omega = dblExtPar[5, 0];
            int nv = dblExtPar.GetLength(0);

            int index = 6;
            foreach (DataRow row in dsGCPCEP.tlbGCP.Rows)
            {
                row["Error"] = dblExtPar[index, 0];
                index++;
            }

  
            ExtPar.Iterations = (int)dblExtPar[nv - 1, 0];
            dsGCPCEP.tlbGCP.WriteXml("~temp.xml");


            XL.Text = ExtPar.XL.ToString();
            YL.Text = ExtPar.YL.ToString();
            ZL.Text = ExtPar.ZL.ToString();
            Kappa.Text = ExtPar.Kappa.ToString();
            Phi.Text = ExtPar.Phi.ToString();
            Omega.Text = ExtPar.Omega.ToString();
            txbIterations.Text = ExtPar.Iterations.ToString();
            txbError.Text = ExtPar.rmsError.ToString();
            tbcOrientation.SelectedTab = tbpOrientation;
  
        }

        private void openToolStripMenuItem_Click(object sender, EventArgs e)
        {
            openFileDialog1.Filter = "Extrinsic Parameters Files|*.Xml";
            if (openFileDialog1.ShowDialog() == DialogResult.OK)
            {
                ExtPar.dsCOP.Clear();
                ExtPar.dsCOP.dtbCOP.ReadXml(openFileDialog1.FileName);
                ExtPar.FromdsCOP();
                XL.Text = ExtPar.XL.ToString();
                YL.Text = ExtPar.YL.ToString();
                ZL.Text = ExtPar.ZL.ToString();
                Kappa.Text = ExtPar.Kappa.ToString();
                Phi.Text = ExtPar.Phi.ToString();
                Omega.Text = ExtPar.Omega.ToString();
                XL0.Text = ExtPar.XL.ToString();
                YL0.Text = ExtPar.YL.ToString();
                ZL0.Text = ExtPar.ZL.ToString();
                Kappa0.Text = ExtPar.Kappa.ToString();
                Phi0.Text = ExtPar.Phi.ToString();
                Omega0.Text = ExtPar.Omega.ToString();
      
                
            }


  
        }

        private void saveToolStripMenuItem_Click(object sender, EventArgs e)
        {
            try
            {
                ExtPar.XL = Convert.ToDouble(XL.Text);
                ExtPar.YL = Convert.ToDouble(YL.Text);
                ExtPar.ZL = Convert.ToDouble(ZL.Text);
                ExtPar.Kappa = Convert.ToDouble(Kappa.Text);
                ExtPar.Phi = Convert.ToDouble(Phi.Text);
                ExtPar.Omega = Convert.ToDouble(Omega.Text);
                ExtPar.dsCOP.Clear();
                ExtPar.TodsCOP();
                if (saveExtParFileDialog1.ShowDialog() == DialogResult.OK)
                {
                    ExtPar.dsCOP.dtbCOP.WriteXml(saveExtParFileDialog1.FileName);
                }
            }
            catch
            {
                MessageBox.Show("Orientation parameters invalid");
            }

        }

        private void exitToolStripMenuItem_Click(object sender, EventArgs e)
        {
            this.Close();
        }
    }
}