using System;
using System.Drawing;
using System.Collections;
using System.ComponentModel;
using System.Windows.Forms;
using System.Data;
using System.Drawing.Drawing2D;

 
namespace OrthoRef
{
    class ZoomPicBox : ScrollableControl
    {
        Image _image;
        [
        Category("Appearance"),
        Description("The image to be displayed")
        ]
        public Image Image
        {
          get{return _image;}
          set
          {
            _image=value;
            UpdateScaleFactor();
            Invalidate();
          }
        }
        bool _invertYaxis;
        [
        Category("Appearance"),
        Description("Invert Y axis.")
        ]
        public bool InvertY
        {
          get{return _invertYaxis;}
          set{_invertYaxis = value;}
        }


        float _zoomy;
        float _zoom=1.0f;
        [
        Category("Appearance"),
        Description("The zoom factor. Less than 1 to reduce. More than 1 to magnify.")
        ]
        public float Zoom
        {
          get{return _zoom;}
          set
          {
            if(value<0 || value<0.00001)
              value=0.00001f;
            _zoom=value;
            UpdateScaleFactor();
            Invalidate();
          }
        }

        public void ZoomExtents()
        {
            if (_image != null)
            {
                float WidthRatio = (float)this.Size.Width / (float)this._image.Width;
                float HeightRatio = (float)this.Size.Height / (float)this._image.Height;
                _zoom = Math.Min(WidthRatio, HeightRatio);
                UpdateScaleFactor();
                Invalidate();
            }
        }

        /// <summary>
        /// Calculates the effective size of the image
        ///after zooming and updates the AutoScrollSize accordingly
        /// </summary>
        private void UpdateScaleFactor()
        {
          if(_image==null)
            this.AutoScrollMinSize=this.Size;
          else
          {
            this.AutoScrollMinSize=new Size(
              (int)(this._image.Width*_zoom+0.5f),
              (int)(this._image.Height * _zoom + 0.5f)
              );
          }
        }
        InterpolationMode _interpolationMode = InterpolationMode.NearestNeighbor;
        [
            Category("Appearance"),
        Description("The interpolation mode used to smooth the drawing")
        ]
        public InterpolationMode InterpolationMode
        {
          get{return _interpolationMode;}
          set{_interpolationMode=value;}
        }


        public Point BacktrackMouse(MouseEventArgs  e)
        /// Back Track the Mouse to return accurate coordinates regardless of zoom or pan effects.
        /// Courtesy of BobPowell.net <seealso cref="http://www.bobpowell.net/backtrack.htm"/>
        /// </summary>
        /// <param name="p">Point to backtrack</param>
        /// <returns>Backtracked point</returns>
        {
            if (_invertYaxis)
                _zoomy = -_zoom;
            else
                _zoomy = _zoom;

            //Creates the drawing matrix with the right zoom;
            Matrix mx = new Matrix(_zoom, 0, 0, _zoomy, 0, 0);
            //pans it according to the scroll bars
            if (_image != null)
               mx.Translate(this.AutoScrollPosition.X / _zoom, this.AutoScrollPosition.Y / _zoomy - this._image.Height * Convert.ToInt32(_invertYaxis));
            //inverts it
            mx.Invert();
            //uses it to transform the current mouse position
            Point[] pa = new Point[] { new Point(e.X, e.Y) };
            mx.TransformPoints(pa);
            return pa[0];
        }

        protected override void OnPaint(PaintEventArgs e)
        {
          //if no image, don't bother
          if(_image==null)
          {
            base.OnPaintBackground(e);
            return;
          }

          if (_invertYaxis)
              _zoomy = -_zoom;
          else
              _zoomy = _zoom;
          
          //Set up a zoom matrix
          Matrix mx=new Matrix(_zoom,0,0,_zoomy,0,0);
          //now translate the matrix into position for the scrollbars
          mx.Translate(this.AutoScrollPosition.X / _zoom, this.AutoScrollPosition.Y / _zoomy - this._image.Height * Convert.ToInt32(_invertYaxis));
          //use the transform
          e.Graphics.Transform=mx;
          //and the desired interpolation mode
          e.Graphics.InterpolationMode=_interpolationMode;
          //Draw the image ignoring the images resolution settings.
          e.Graphics.DrawImage(_image, new Rectangle(0, 0, this._image.Width, this._image.Height), 0, 0, _image.Width, _image.Height, GraphicsUnit.Pixel);
          base.OnPaint (e);
        }


        public ZoomPicBox()
        {
          //Double buffer the control
          this.SetStyle(ControlStyles.AllPaintingInWmPaint |
            ControlStyles.UserPaint |
            ControlStyles.ResizeRedraw |
            ControlStyles.UserPaint |
            ControlStyles.DoubleBuffer, true);
          this.AutoScroll=true;
        }
    }
}
